#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Metadata
__author__     = "Maximilian Golla"
__credits__    = ["Philipp Markert", "Daniel V. Bailey", "Maximilian Golla", "Markus Duermuth", "Adam J. Aviv"]
__license__    = "The MIT License (MIT)"
__version__    = "1.0.0, 2020-03-11"
__maintainer__ = "Maximilian Golla"
__email__      = "maximilian.golla@csp.mpg.de"
__status__     = "Development"

import sys, datetime, itertools

# aaaa
# 10 PINs
def p0(pin):
    if pin[0] == pin[1] == pin[2] == pin[3]:
        return True
    return False

# abab
# 100 PINs
def p1(pin):
    if pin[0] == pin[2] and pin[1] == pin[3]:
        return True
    return False

# aabb
# 100 PINs
def p2(pin):
    if pin[0] == pin[1] and pin[2] == pin[3]:
            return True
    return False

# years
# 60 PINs
def year(pin):
    pin = int(pin)
    if pin >= 1956 and pin <= 2015:
        return True
    return False

# common
# 24 PINs
'''
Joseph Bonneau, Soeren Preibusch, and Ross Anderson.
A Birthday Present Every Eleven Wallets? The Security of Customer-Chosen Banking PINs.
In Financial Cryptography and Data Security, FC ’12, pages 25–40, Kralendijk, Bonaire, February 2012. Springer.
'''
def common(pin):
    common = [                          '0102', '0103', # Bonneau et al.
                        '0110', '0111',                 # Bonneau et al.
                                                '0123', # Bonneau et al.
                '1004',                                 # Not in Bonneau et al.
                                '1101', '1102', '1103', # Bonneau et al.
                        '1110',         '1112',         # Bonneau et al.
                                                '1123', # Bonneau et al.
                                '1201', '1202', '1203', # Bonneau et al.
                        '1210', '1211',                 # Bonneau et al.
                '1234',                                 # Bonneau et al. series
                '2229',                                 # Bonneau et al. "baby"
                '2580',                                 # Bonneau et al. keypad walk
                '4321',                                 # Not in Bonneau et al. (series)
                '5683',                                 # Bonneau et al. "love"
                '7465',                                 # Bonneau et al. "pink"
                '7667']                                 # Bonneau et al. "poop"

    if pin in common:
        return True
    return False

def check_pin(pin):
    result = []
    if p0(pin):
        result.append('p0 (aaaa)')
    if p1(pin):
        result.append('p1 (abab)')
    if p2(pin):
        result.append('p2 (aabb)')
    if year(pin):
        result.append('year')
    if common(pin):
        result.append('common')
    return result

# Generates all theoretically possible PINs == 10000
def generate():
    allpins = []
    pin_length = 4
    alphabet = [0,1,2,3,4,5,6,7,8,9]
    for candidate in itertools.product(alphabet, repeat=pin_length):
        pin = ''.join(str(i) for i in candidate)
        allpins.append(pin)
    return allpins

def main():
    generated = generate()
    blacklisted = 0
    for pin in generated:
        blacklist_check = check_pin(pin)
        if len(blacklist_check) != 0:
            blacklisted += 1
            print(pin, blacklist_check)
    print("blacklisted:", blacklisted, "({} %)".format((blacklisted*100.0)/len(generated)))

if __name__ == '__main__':
    main()
